/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Klasse zur Ermittlung der Information, ob ein Konfigurationsbereich Modell- und/oder Objekt-Definitionen besitzt. Die
 * Bestimmung erfolgt ueber das Datenmodell und den zum Konfigurationsbereich gehoerenden Objekte.
 * 
 * @author Pittner
 * 
 */
public class KonfigBereichBestimmeModellObjekt
{
  /**
   * Das Datenmodell
   */
  DataModel _dataModel = null;

  /**
   * zu pruefender Konfigurationsbereich
   */
  ConfigurationArea _configArea = null;

  /**
   * Liste von Pids aller Typen und Mengen des Meta-Modells
   */
  private final static String[] pidsMetaModellTypen = new String[] { "typ.typ", "typ.attributListenDefinition",
      "typ.attributTyp", "typ.attributgruppe", "typ.attribut", "typ.attributgruppenVerwendung", "typ.werteZustand",
      "typ.mengenVerwendung", "typ.werteBereich", "typ.aspekt", "menge.attribute", "menge.mengenVerwendungen",
      "menge.attributgruppenVerwendungen", "menge.objektTypen", "menge.attributgruppen", "menge.werteZustaende" };

  /**
   * Flagge, die anzeigt ob der Konfigurationsbereich Modell-Definitionen hat
   */
  Boolean isModell = false;

  /**
   * Flagge, die anzeigt ob der Konfigurationsbereich Objekt-Definitionen hat
   */
  Boolean isObjekt = false;

  /**
   * Konstruktor der Klasse. Die uebergebenen Parameter werden auf null geprueft.
   * 
   * @param dataModel
   *          das Datenmodell
   * @param configArea
   *          der zu pruefende Konfugurationbereich
   */
  public KonfigBereichBestimmeModellObjekt(DataModel dataModel, ConfigurationArea configArea)
  {
    if (dataModel != null && configArea != null)
    {

      _dataModel = dataModel;
      _configArea = configArea;

      bestimmeModellObjektAusKonfigBereich();

    }
    else
      throw new IllegalArgumentException("Argument ist null");

  }

  /**
   * Methode die den Konfigbereich auf Modell- sowie Objektdefinitioen prueft.
   */
  private void bestimmeModellObjektAusKonfigBereich()
  {
    List<SystemObjectType> listMetaModelTypes = new ArrayList<SystemObjectType>();

    for (int i = 0; i < pidsMetaModellTypen.length; i++)
    {
      SystemObjectType type = _dataModel.getType(pidsMetaModellTypen[i]);
      listMetaModelTypes.add(type);
    }

    Collection<SystemObject> currentObjects = _configArea.getCurrentObjects();

    Collection<SystemObject> objectsFromMetaModelTypes = _configArea.getObjects(listMetaModelTypes,
        ObjectTimeSpecification.valid());

    if (objectsFromMetaModelTypes.size() > 0)
    {
      isModell = true;
    }

    if (currentObjects.size() > (objectsFromMetaModelTypes.size() + 1))
    {
      isObjekt = true;
    }

  }

  /**
   * Holt die Info, ob der Konfigurationsbereich Modell-Definitionen besitzt.
   * 
   * @return Info ob Modell-Definitionen vorhanden
   */
  public Boolean getIsModell()
  {
    return isModell;
  }

  /**
   * Holt die Info, ob der Konfigurationsbereich Objekt-Definitionen besitzt.
   * 
   * @return Info ob Objekt-Definitionen vorhanden
   */
  public Boolean getIsObjekt()
  {
    return isObjekt;
  }

}
