/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

/**
 * Es werden Informationen ber eine Attributgruppe modelliert, die von einem Hierarchieobjekttypen 
 * spezifiziert ist
 * 
 * @author Sans
 *
 */
public class AtgInfo
{
  /**
   * Name der Attributgruppe
   */
  private String _atg;
  
  /**
   * Ist diese Attributgruppe optional?
   */
  private boolean _isOptional;
  
  /**
   * Ist diese Attributgruppe nicht optional sein?
   */
  private final boolean _isMandatory;
  
  /**
   * Liste der Attribute (Namen), die von dieser Attributgruppe benutzt werden
   */
  private List<String> _attribute = new Vector<String>();
  
  /**
   * Map, enthlt fr jeden Attributnamen die Liste der Attributbeschreibungen
   */
  private Map<String, IAttributInfo> _attributInfos = new HashMap<String, IAttributInfo>();
  
//  public Map<String, IAttributInfo> getAttributInfos()
//  {
//    return _attributInfos;
//  }

  /**
   * Erzeugt eine neue Instanz vom Typ AtgInfo
   * 
   * @param atg Name der Attributgruppe
   * @param isMandatory ist diese Attributgruppe nicht optional?
   * @param attributInfos Liste der Attribute (Namen), die von dieser Attributgruppe benutzt werden
   */
  public AtgInfo(String atg, boolean isMandatory, IAttributInfo[] attributInfos)
  {
    _isMandatory = isMandatory;
    _isOptional = !isMandatory;
    
    _atg = atg;
    
    if (attributInfos != null)
    {
      for (IAttributInfo info: attributInfos)
      {
        _attribute.add(info.getName());
        _attributInfos.put(info.getName(), info);
      }
    }
  }

  /**
   * Ist diese Attributgruppe optional ?
   * 
   * @return true, wenn diese Attributgruppe optional ist, false sonst - insbesondere
   * wenn, wenn diese Instanz nicht optional ist (isMandatatory() liefert true)
   */
  public boolean isOptional()
  {
    return _isOptional;
  }

  /**
   * Setzt eine Attributgruppe auf optional. Dies geschieht nur, wenn diese Instanz nicht optional ist 
   * (isMandatatory() liefert true)
   * 
   * @param isOptional Flag, das anzeigt ob diese Instanz optional sein soll oder nicht
   * 
   * @return true, wenn der neue Zustand gesetzt werden konnte, false sonst - insbesondere
   * wenn, wenn diese Instanz nicht optional ist (isMandatatory() liefert true)
   */
  public boolean setOptional(boolean isOptional)
  {
    if (isOptional)
    {
      if (_isMandatory)
      {
        return false;
      }
      else
      {
        _isOptional = isOptional;
        return true;
      }
    }
    else
    {
      _isOptional = isOptional;
      return true;
    }
  }
  
  /**
   * Ist diese Instanz nicht optional
   * 
   * @return true, wenn diese Instanz nicht optional ist, false sonst
   */
  public boolean isMandatory()
  {
    return _isMandatory;
  }

  /**
   * Bestimmt den Name der Attributgruppe
   * 
   * @return den Namen der Attributgruppe
   */
  public String getAtg()
  {
    return _atg;
  }
  
  /**
   * Bestimmt die Namen der benutzten Attribute der Attributgruppe
   * 
   * @return die Liste der Namen der benutzten Attribute der Attributgruppe
   */
  public List<String> getAttribute()
  {
    return Collections.unmodifiableList(_attribute);
  }
  
  /**
   * Bestimmt die Beschreibung eines benutzten Attributs der Attributgruppe
   * 
   * @param feld der Name des Attributs
   * @return die Attributbeschreibung
   */
  public IAttributInfo getAttributInfo(String feld)
  {
    return _attributInfos.get(feld);
  }

  /**
   * Bestimmt die Beschreibung eines Attributs, das nicht direkt benutzt wird,
   * sondern innerhalb eines benutzten Attributs (verschachtelt) als inneres
   * Attribut vorkommt
   * 
   * @param feld der Name des Attributs
   * @return die Attributbeschreibung
   */
  public IAttributInfo getAttributInfoTiefe(String feld)
  {
    IAttributInfo info = _attributInfos.get(feld);
    
    if (info == null)
    {
      for (IAttributInfo attInfo : _attributInfos.values())
      {
        if (attInfo.hasInnereAttribute())
        {
          List<IAttributInfo> innereAttribute = attInfo.getInnereAttribute();
          info = getAttributInfoRekursiv(feld, innereAttribute);
          if (info != null)
            return info;
        }
      }
    }
    
    return info;
  }

  /**
   * Interne Methode zur rekursiven Bestimmung der Beschreibung eines Attributs, das nicht 
   * direkt benutzt wird, sondern innerhalb eines benutzten Attributs (verschachtelt) als 
   * inneres Attribut vorkommt
   * 
   * @param attInfoListe der Name des Attributs
   * @return Liste von Attributbeschreibungen
   */
  private IAttributInfo getAttributInfoRekursiv(String feld, List<IAttributInfo> attInfoListe)
  {
    IAttributInfo info = null;
    
    if (attInfoListe != null)
    {
      for (int i = 0; i < attInfoListe.size(); i++)
      {
        IAttributInfo attInfo = attInfoListe.get(i);
        if (attInfo.getName().equals(feld))
        {
          info = attInfo;
          if (info != null)
            return info;
        }
        if (attInfo.hasInnereAttribute())
        {
          List<IAttributInfo> innereAttribute = attInfo.getInnereAttribute();
          info = getAttributInfoRekursiv(feld, innereAttribute);
          if (info != null)
            return info;
        }
      }
    }
    return info;
  }
  
}
