/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse KBHierarchie modelliert eine Hierarchie, die in ihrer ersten Hierarchieebene
 * ausschliesslich Systemobjekte vom Typ ConfigurationArea, also Konfigurationsbereiche,
 * enthlt. Alle untergeordneten Ebenene enthalten ausschliesslich Objekte vom Typ 
 * Hierarchieobjekt.
 * In dieser Klasse wird ber Instanzen vom Typ Comparator die Sortierung der Hierarchieobjekte und auch 
 * der Konfigurationsbereiche definiert.
 * 
 * @author Sans
 *
 */
public class KBHierarchie implements IHierarchie
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Statisches Flag zum Ein-und Ausschalten des Loggings fr Benachrichtigungen des Listeners
   * vom Typ IKBHierarchieBeobachter  
   */
  private final static boolean SYS_OUT = false;
  
  /**
   * Liste der Konfigurationsbereiche
   */
  private List<ConfigurationArea> _konfigurationsBereiche = new Vector<ConfigurationArea>();
  
  /**
   * Datenmodell der Konfiguration
   */
  private ConfigDataModel _dataModel;
  
  public ConfigDataModel getDataModel()
  {
    return _dataModel;
  }
  
  /**
   * Bestimmt die Konfigurationsbereiche in der ersten Ebene dieser Hierarchie
   * 
   * @return die Liste der Konfigurationsbereiche in der ersten Ebene dieser Hierarchie
   */
  public List<ConfigurationArea> getKonfigurationsBereiche()
  {
    return Collections.unmodifiableList(_konfigurationsBereiche);
  }
  
  /**
   * Map, enthlt fr jedes in der Hierarchie enthaltene Objekt die Liste der untergeordenten Objekte
   */
  private Map<Object, List<HierarchieObjekt>> _untergeordneteObjekte = 
    new HashMap<Object, List<HierarchieObjekt>>();
  
  /**
   * Map, enthlt fr jedes in der Hierarchie enthaltene Hierarchieobjekt das bergeordnete Objekt 
   */
  private  Map<HierarchieObjekt, Object> _uebergeordneteObjekte = 
    new HashMap<HierarchieObjekt, Object>();
  
  
  public List<HierarchieObjekt> getUntergeordneteObjekte(Object objekt)
  {
    List<HierarchieObjekt> result = _untergeordneteObjekte.get(objekt);
    
    if (result == null)
    {
      result = Collections.emptyList();
    }
    
    return result;
  }
  
  /**
   * Die diese Hierarchie benutztende IHierarchieManager-Instanz
   */
  private IHierarchieManager _hm;
    
  public Object getUebergeordnetesObjekt(HierarchieObjekt ho)
  {
    return _uebergeordneteObjekte.get(ho);
  }
  
  /**
   * Erzeugt eine neue Instanz vom Typ KBHierarchie
   * 
   * @param dataModel das Datenmodell zum Zugriff auf die Konfiguration
   * @param hm die IHierarchieManager-Instanz, die fr die Manipulation dieser Hierarchie vorgesehen ist
   */
  public KBHierarchie(ConfigDataModel dataModel, IHierarchieManager hm)
  {
    _dataModel = dataModel;
    _hm = hm;
    
    _hm.setHierarchie(this);
  }
  
  /**
   * Fgt eine Konfigurationsbereich in die erste Ebene dieser Hierarchie ein
   * 
   * @param kb der einzufgende Konfigurationsbereich
   */
  public void kbHinzufuegen(ConfigurationArea kb)
  {
    int index = Collections.binarySearch(_konfigurationsBereiche, kb, comparatorSOAufsteigend());
    if (index < 0) // Nur dann ist der kb noch nicht in der Liste enthalten
    {
      _konfigurationsBereiche.add(-index - 1, kb);
      _hm.bestimmeUntergeordneteHOs(kb);
    }  
  }
  
  /**
   * Entfernt eine Objekt - Konfigurationsbereich oder Hierarchieobjekt rekursiv mit
   * allen untergeordneten Objekten aus dieser Hierarchie
   * 
   * @param objekt das zu entferndende Objekt
   */
  public void objektEntfernen(Object objekt)
  {
    List<HierarchieObjekt> liste = _untergeordneteObjekte.get(objekt);
    if (liste != null)
    {
      for (HierarchieObjekt ho: liste)
      {
        objektEntfernen(ho);
      }
    }
    
    if (objekt instanceof HierarchieObjekt)
    {
      Object vater = _uebergeordneteObjekte.get(objekt);
      if (vater != null)
      {  
        objektEntfernen(vater, (HierarchieObjekt)objekt);
      }  
    }
    else if (objekt instanceof ConfigurationArea)
    {
      _konfigurationsBereiche.remove((ConfigurationArea)objekt);
    }
  }
  
  public void objektHinzufuegen(Object vater, HierarchieObjekt kind)
  {
    objektHinzufuegenIntern(vater, kind);
    
    notifyHOHinzufgen(vater, kind);    
  }
  
  /**
   * Intern benutzte Methode zum Einfgen eine Hierarchieobjekts in die Hierarchie
   * unterhalb eines bergeordneten Objekts
   * 
   * @param vater das bergeordnete Objekts, unter dem das Hierarchieobjekt eingefgt werden soll
   * @param kind das einzufgende Hierarchieobjekt
   */
  private void objektHinzufuegenIntern(Object vater, HierarchieObjekt kind)
  {
    List<HierarchieObjekt>liste = _untergeordneteObjekte.get(vater);
    
    if (liste == null)
    {
      liste = new Vector<HierarchieObjekt>();
      _untergeordneteObjekte.put(vater, liste);
    }
    
    int index = Collections.binarySearch(liste, kind, comparatorHOAufsteigend());
    liste.add(index < 0 ? -index - 1 : index, kind);
    
    _uebergeordneteObjekte.put(kind, vater);
  }
  
  public void objektEntfernen(Object vater, HierarchieObjekt kind)
  {
    objektEntfernenIntern(vater, kind);
    
    notifyHOEntfernen(vater, kind);
  }
  
  /**
   * Intern benutzte Methode zum Entfernen eine Hierarchieobjekts in die Hierarchie
   * unterhalb eines bergeordneten Objekts
   * 
   * @param vater das bergeordnete Objekts, unter dem das Hierarchieobjekt entfernt werden soll
   * @param kind das zu entferndende Hierarchieobjekt
   */
  private void objektEntfernenIntern(Object vater, HierarchieObjekt kind)
  {
    List<HierarchieObjekt>liste = _untergeordneteObjekte.get(vater);
    
    if (liste != null)
    {
      liste.remove(kind);
    }
    
    Object objekt = _uebergeordneteObjekte.get(kind);
    if (objekt == vater)
    {
      _uebergeordneteObjekte.remove(kind);
    }
  }
  
  
  // Sortierung 
  
  /**
   * Bestimmt eine Comparator-Instanz fr die aufsteigende Sortierung von Systemobjekten  
   * @return eine Comparator-Instanz fr die aufsteigende Sortierung von Systemobjekten
   */
  public Comparator<SystemObject> comparatorSOAufsteigend()  
  {
    return _comparatorSOAufsteigend; 
  }
   
  private static Comparator<SystemObject> _comparatorSOAufsteigend = 
    new Comparator<SystemObject>()
    {
      public int compare(SystemObject so1, SystemObject so2)
      {
        String string1 =  so1 != null ? so1.getPid() : ""; 
        String string2 =  so2 != null ? so2.getPid() : "";
        return String.CASE_INSENSITIVE_ORDER.compare(string1, string2);
      }  
    };
  
  /**
   * Bestimmt eine Comparator-Instanz fr die absteigende Sortierung von Systemobjekten  
   * @return eine Comparator-Instanz fr die absteigende Sortierung von Systemobjekten
   */
  public Comparator<SystemObject> comparatorSOAbsteigend()  
  {
    return _comparatorSOAbsteigend; 
  }
   
  private static Comparator<SystemObject> _comparatorSOAbsteigend = 
    new Comparator<SystemObject>()
    {
      public int compare(SystemObject so1, SystemObject so2)
      {
        String string1 =  so1 != null ? so1.getPid() : ""; 
        String string2 =  so2 != null ? so2.getPid() : "";
        return String.CASE_INSENSITIVE_ORDER.compare(string2, string1);
      }  
    };  
    
  private int compareHOTypen(HierarchieObjekt ho1, HierarchieObjekt ho2)
  {
//    if (getUebergeordnetesObjekt(ho1) != getUebergeordnetesObjekt(ho2))
//    {
//      throw new IllegalStateException("Unterschiedliche bergeordnete Objekte fr zu vergleichende Objekte " + ho1 + ", " + ho2);
//    }
    
    List<IHierarchieObjektTyp> folge1 = ho1.getTypSortierFolge();
    List<IHierarchieObjektTyp> folge2 = ho2.getTypSortierFolge();
    
    if (!folge1.equals(folge2))
    {
      throw new IllegalStateException("Typ-Sortierfolgen sind ungleich fr zu vergleichende Objekte " + ho1 + ", " + ho2);
    }  
     
    int index1 = folge1.indexOf(ho1.getHierarchieObjektTyp());
    if (index1 < 0)
    {
      throw new IllegalStateException("Objekttyp (HOT) " + ho1.getHierarchieObjektTyp() + " ist nicht in der Typ-Sortierfolge enthalten fr " + ho1);
    }
    
    int index2 = folge2.indexOf(ho2.getHierarchieObjektTyp());
    if (index2 < 0)
    {
      throw new IllegalStateException("Objekttyp (HOT) " + ho2.getHierarchieObjektTyp() + " ist nicht in der Typ-Sortierfolge enthalten fr " + ho2);
    }
    
    if (index1 < index2)
    {
      return -1;
    }
    else if (index1 > index2)
    {
      return 1;
    }
    else 
    {
      return 0; // index 1 == index 2: Typen sind gleich
    }
  }    
  
  private Comparator<HierarchieObjekt>  _comparatorHOAufsteigend = 
    new Comparator<HierarchieObjekt>()
    {
      public int compare(HierarchieObjekt ho1, HierarchieObjekt ho2)
      {
        int result = compareHOTypen(ho1, ho2);
        if (result == 0)
        {
          result = _comparatorSOAufsteigend.compare(ho1.getSystemObjekte().get(0),
              ho2.getSystemObjekte().get(0));
        }
        return result;
      }  
    };
    
  /**
   * Bestimmt eine Comparator-Instanz fr die aufsteigende Sortierung von Hierarchieobjekten  
   * @return eine Comparator-Instanz fr die aufsteigende Sortierung von Hierarchieobjekten
   */
  public Comparator<HierarchieObjekt> comparatorHOAufsteigend()  
  {
    return _comparatorHOAufsteigend; 
  }
      
  private Comparator<HierarchieObjekt>  _comparatorHOAbsteigend = 
    new Comparator<HierarchieObjekt>()
    {
      public int compare(HierarchieObjekt ho1, HierarchieObjekt ho2)
      {
        int result = 0; //compareHOTypen(ho1, ho2);
        if (result == 0)
        {
          result = _comparatorSOAbsteigend.compare(ho1.getSystemObjekte().get(0),
              ho2.getSystemObjekte().get(0));
        }
        return result;
      }  
    };
      
  /**
   * Bestimmt eine Comparator-Instanz fr die absteigende Sortierung von Hierarchieobjekten  
   * @return eine Comparator-Instanz fr die absteigende Sortierung von Hierarchieobjekten
   */    
  public Comparator<HierarchieObjekt> comparatorHOAbsteigend()  
  {
    return _comparatorHOAbsteigend; 
  }  
  
  public int getAnzahlEbenen()
  {
    int result = 0;
    
    for (Object kb: getKonfigurationsBereiche())
    {
      int ebenen = getAnzahlEbenen(kb, 1);
      if (ebenen > result)
      {
        result = ebenen;
      }
    }
    
    return result;
  }
  
  /**
   * Bestimmt die Anzahl der Ebenen unterhalb eines Objekts in dieser Hierachie
   * 
   * @param objekt das Objekt, unter dem die Ebenen bestimmt werden sollen
   * @param ebene die Ebene des bergebenen Objekts
   * @return die Anzahl der Ebenen unter der bergebenene Objekt
   */
  private int getAnzahlEbenen(Object objekt, int ebene)
  {
    int result = ebene;
    
    for (Object ho: getUntergeordneteObjekte(objekt))
    {
      int ebenen = getAnzahlEbenen(ho, ebene + 1);
      if (ebenen > result)
      {
        result = ebenen;
      }
    }
    
    return result;
  }
  
  public boolean alleObjekteAktiviert(HierarchieObjekt ho, boolean rekursiv)
  {
    for (SystemObject so: ho.getSystemObjekte())
    {
      ConfigurationObject ko = (ConfigurationObject)so;
      ConfigurationArea kb = ko.getConfigurationArea();
      
      if (ko.getValidSince() > kb.getActiveVersion())
      {
        return false;
      }
    }
    
    if (rekursiv)
    {
      for (HierarchieObjekt kind: getUntergeordneteObjekte(ho))
      {
        if (!alleObjekteAktiviert(kind, true))
        {
          return false;
        }
      }
    }
    
    return true;
  }
  
  /**
   * Bestimmt den Konfigurationsbereich fr ein Objekt der Hierarchie
   * 
   * @param objekt das Obejkt in der Hierarchie, fr das der KB bestimmt werden soll
   * @return den KB des bergebenen Objekts oder <code>null</code>, falls das Objekt nicht zur
   * Hierarchie gehrt
   */
  public ConfigurationArea getKonfigurationsBereich(Object objekt)
  {
    Object lauf = objekt;
    
    while (lauf instanceof HierarchieObjekt)
    {
      lauf = getUebergeordnetesObjekt((HierarchieObjekt)lauf);
    }
    
    return lauf instanceof ConfigurationArea ? (ConfigurationArea)lauf : null;
  }
   
  /**
   * Menge der auf nderungen an dieser Hierarchie angemeldenten Instanzen vom Typ 
   * IKBHierarchieBeobachter
   */
  private Set<IHierarchieBeobachter> _beobachter = new LinkedHashSet<IHierarchieBeobachter>();
  
  /**
   * Meldet eine IKBHierarchieBeobachter-Instanz auf nderungen an 
   * 
   * @param beobachter anzumeldende IKBHierarchieBeobachter-Instanz
   */  
  public void beobachterAnmelden(IHierarchieBeobachter beobachter)
  {
    _beobachter.add(beobachter);
  }
      
  /**
  * Meldet eine angemeldete IKBHierarchieBeobachter-Instanz ab
  * 
  * @param beobachter abzumeldende IKBHierarchieBeobachter-Instanz
  */
  public void beobachterAbmelden(IHierarchieBeobachter beobachter)
  {
    _beobachter.remove(beobachter);
  }
  
  /**
   * Informiert alle angemeldeten IKBHierarchieBeobachter-Instanzen ber eine
   * nderung
   * 
   * @param feldName der Name des Eingabefeldes
   * @param wert der genderte Wert des Eingabefeldes
   */
  protected void alleBeobachterInformieren(IHierarchieBeobachter.EVENT event, Object vater, 
      HierarchieObjekt ho)
  {
    ConfigurationArea kb = getKonfigurationsBereich(vater);
    
    for (IHierarchieBeobachter beobachter: _beobachter)
    {
      beobachter.hierarchieAktualisiert(event, kb, vater, ho);
    }
  }
  
  public void notifyHOGeaendert(HierarchieObjekt ho)
  {
    Object vater = getUebergeordnetesObjekt(ho);
    if (vater != null)
    {
      if (SYS_OUT) System.out.println("NOTIFY: Aktualisieren HO " + ho);
      
      alleBeobachterInformieren(IHierarchieBeobachter.EVENT.OBJEKT_AKTUALISIERT, vater, ho);
    }
    else
    {
      _debug.warning("Kein Eltern-Objekt zu " + ho + " in der Hierarchie gefunden");
    }
  }
 
  /**
   * Informiert ggf. angemeldete Beobachter ber das Hinzufgen eines Hierarchieobjekts
   * 
   * @param vater das Objekt, dem das bergebene Hierarchieobjekt als untergeordnetes Objekt
   * hinzugefgt wird
   * @param ho das hinzuzufgende Hierarchieobjekt
   */
  private void notifyHOHinzufgen(Object vater, HierarchieObjekt kind)
  {
    if (SYS_OUT) System.out.println("NOTIFY: Hinzufgen HO " + kind + " zu Vater " + vater);
    
    alleBeobachterInformieren(IHierarchieBeobachter.EVENT.OBJEKT_NEU, vater, kind);
  }
  
  /**
   * Informiert ggf. angemeldete Beobachter ber das Entfernen eines Hierarchieobjekts
   * 
   * @param vater das dem zu entfernenden Hierarchieobjekt bergeordnete Objekt
   * @param ho das zu entfernende Hierarchieobjekt
   */
  private void notifyHOEntfernen(Object vater, HierarchieObjekt kind)
  {
    if (SYS_OUT) System.out.println("NOTIFY: Entfernen HO " + kind + " zu Vater " + vater);
    
    alleBeobachterInformieren(IHierarchieBeobachter.EVENT.OBJEKT_ENTFERNT, vater, kind);
  }

  /**
   * Informiert ggf. angemeldete Beobachter ber das Verschieben eines Hierarchieobjekts
   * 
   * @param vater das Objekt, zu dem das bergebene Hierarchieobjekt als neues bergeordnetes
   * Objekt verschoben wird
   * @param ho das zu verschiebende Hierarchieobjekt
   */
  private void notifyHOVerschieben(Object vater, HierarchieObjekt kind)
  {
    if (SYS_OUT) System.out.println("NOTIFY: Verschieben HO " + kind + " von Vater " + vater +
        " zu neuem Vater " + getUebergeordnetesObjekt(kind));
    
    alleBeobachterInformieren(IHierarchieBeobachter.EVENT.OBJEKT_VERSCHOBEN, vater, kind);
  }
  
  public void objektVerschieben(Object alterVater, Object neuerVater, HierarchieObjekt kind)
  {
    objektHinzufuegenIntern(neuerVater, kind);
    objektEntfernenIntern(alterVater, kind);
    
    notifyHOVerschieben(alterVater, kind);
  }
  
}
