/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Bildet eine Teilhierarchie einer Instanz vom Typ IHierarchieManager in einer 
 * temporren Hierarchie ab, und erlaubt das Lschen der Teilhierarchie, wobei 
 * die Daten der gelschten Hierarchieobjekte zwischengespeichert werden
 * 
 * @author Sans
 *
 */
public class TempHierarchie
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Wurzelobjekt der Hierachie
   */
  HierarchieObjekt _wurzel = null;
    
  /**
   * Bestimmt das Wurzelobjekt der Hierarchie
   * 
   * @return as Wurzelobjekt der Hierarchie
   */
  public HierarchieObjekt getWurzel()
  {
    return _wurzel;
  }
  
  /**
   * IHierarchieManager-Instanz, fr dessen Hierarchie diese Hierarchie erzeugt wurde
   */
  private IHierarchieManager _hm;
  
  /**
   * Map, enthlt fr jedes in der Hierarchie enthaltene Objekt die Liste der untergeordenten Objekte
   */
  private Map<HierarchieObjekt, List<HierarchieObjekt>> _untergeordneteObjekte = 
    new HashMap<HierarchieObjekt, List<HierarchieObjekt>>();
    
  /**
   * Map, enthlt fr jedes in der Hierarchie enthaltene Hierarchieobjekt das bergeordnete Objekt 
   */
  private  Map<HierarchieObjekt, HierarchieObjekt> _uebergeordneteObjekte = 
    new HashMap<HierarchieObjekt, HierarchieObjekt>();
    
  /**
   * Bestimmt die untergeordneten Hierarchieobjekte zu einem Objekt der Hierarchie
   * 
   * @param ho das Objekt, zu dem die untergordenten Objekte bestimmt werden sollen
   * @return Liste der untergeordneten Objekte zum bergebenen Objekt
   */
  public List<HierarchieObjekt> getUntergeordneteObjekte(HierarchieObjekt ho)
  {
    List<HierarchieObjekt> result = _untergeordneteObjekte.get(ho);
    
    if (result == null)
    {
      result = Collections.emptyList();
    }
    
    return result;
  }
  
  /**
   * Bestimmt das bergeordnete Objekt zu einem Hierachie
   * 
   * @param ho das Hierarchieobjekt, zu dem das bergeordnete Objekt bestimmt werden soll
   * @return das bergeordnete Objekt zum bergebenen Hierarchieobjekt
   */
  public HierarchieObjekt getUebergeordnetesObjekt(HierarchieObjekt ho)
  {
    return _uebergeordneteObjekte.get(ho);
  }
    
  /**
   * Erzeugt eine neue Instanz vom Typ HOHierarchie aus Hierarchieobjekten einer Instanz
   * vom Typ IHierarchie
   * Es werden das bergebenen Wurzelobjekt und alle untergeordenten Objekte in die neue
   * Instanz bernommen
   * 
   * @param wurzel das Wuurzelobjekt
   * @param hierarchie die Hierachie, die das Wurzelobjekt und die zu bernehmenden 
   * Hierachieobjekte enthlt
   */
  public TempHierarchie(HierarchieObjekt wurzel, IHierarchieManager hm)
  {
    _wurzel = wurzel;
    _hm = hm;
    
    hinzufuegen(wurzel, hm.getHierarchie());
  }

  /**
   * Fgt ein Hierachieobjekt und rekursiv alle seine untergeordneten Objekte aus einer
   * Instanz vom Typ IHierarchie dieser Hierarchie hinzu
   * 
   * @param ho das hinzuzufgende Objekt
   * @param hierarchie die Hierarchie, die das hinzuzufgende Objekt enthlt
   */
  private void hinzufuegen(HierarchieObjekt ho, IHierarchie hierarchie)
  {
    for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte( ho ))
    {
      objektHinzufuegen( ho, kind );
      hinzufuegen(kind, hierarchie);
    }
  }
    
  /**
   * Entfernt ein Hierarchieobjekt rekursiv mit allen untergeordneten Objekten aus 
   * dieser Hierarchie
   * 
   * @param ho das zu entferndende Objekt
   */
  public void objektEntfernen(HierarchieObjekt ho)
  {
    List<HierarchieObjekt> liste = _untergeordneteObjekte.get(ho);
    if (liste != null)
    {
      for (HierarchieObjekt kind: liste)
      {
        objektEntfernen(kind);
      }
    }
    
    HierarchieObjekt vater = _uebergeordneteObjekte.get(ho);
    if (vater != null)
    {  
      objektEntfernen(vater, ho);
    }  
  }
  
  /**
   * Intern benutzte Methode zum Einfgen eine Hierarchieobjekts in die Hierarchie
   * unterhalb eines bergeordneten Objekts
   * 
   * @param vater das bergeordnete Objekts, unter dem das Hierarchieobjekt eingefgt werden soll
   * @param kind das einzufgende Hierarchieobjekt
   */
  private void objektHinzufuegen(HierarchieObjekt vater, HierarchieObjekt kind)
  {
    List<HierarchieObjekt>liste = _untergeordneteObjekte.get(vater);
    
    if (liste == null)
    {
      liste = new Vector<HierarchieObjekt>();
      _untergeordneteObjekte.put(vater, liste);
    }
    
    liste.add(kind);
    
    _uebergeordneteObjekte.put(kind, vater);
  }
  
  /**
   * Intern benutzte Methode zum Entfernen eine Hierarchieobjekts in die Hierarchie
   * unterhalb eines bergeordneten Objekts
   * 
   * @param vater das bergeordnete Objekts, unter dem das Hierarchieobjekt entfernt werden soll
   * @param kind das zu entferndende Hierarchieobjekt
   */
  private void objektEntfernen(Object vater, HierarchieObjekt kind)
  {
    List<HierarchieObjekt>liste = _untergeordneteObjekte.get(vater);
    
    if (liste != null)
    {
      liste.remove(kind);
    }
    
    Object objekt = _uebergeordneteObjekte.get(kind);
    if (objekt == vater)
    {
      _uebergeordneteObjekte.remove(kind);
    }
  }
 
  public IStatus zwischenspeichernUndLoeschen(HierarchieObjekt ho)
  {
    MultiStatus status = new MultiStatus(Activator
        .PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho +
        " vom Typ " + ho.getHierarchieObjektTyp(), null);
    
    IHierarchie hierarchie = _hm.getHierarchie(); 
    
    while (!hierarchie.getUntergeordneteObjekte(ho).isEmpty())
    {
      HierarchieObjekt kind = hierarchie.getUntergeordneteObjekte(ho).get(0);
      
      status.add(zwischenspeichernUndLoeschen(kind));
      
      if (status.matches(IStatus.CANCEL | IStatus.ERROR))
      {
        break;
      }
    }
    
    erzeugeZwischengespeicherteDaten(ho);
    
    if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
    {  
      status.merge(_hm.loescheHO(ho));
    }
    
    return status;
  }
  
  private void erzeugeZwischengespeicherteDaten(HierarchieObjekt ho)
  {
  // Erzeuge temporre Daten
    HODaten hoDaten = new HODaten();
    for (SystemObject so: ho.getSystemObjekte())
    {
      SystemObjektDaten soDaten = new SystemObjektDaten();
      soDaten.setPid(so.getPid());
      soDaten.setName(so.getName());
            
      ConfigDataModel dataModel = _hm.getHierarchie().getDataModel();
      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
      
      // ATG-Daten
      for (AttributeGroupUsage atgu: so.getUsedAttributeGroupUsages())
      {
        AttributeGroup atg = atgu.getAttributeGroup();
        Data data = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
        if (data != null)
        {
          soDaten.setAtgDaten(atg.getPid(), data);
        }
      }
      
      // Mengen
      for (ObjectSet menge: ((ConfigurationObject)so).getObjectSets())
      {
        if (menge instanceof NonMutableSet)
        {
          soDaten.setMengenElemente(menge.getName(), ((NonMutableSet)menge).getElementsInModifiableVersion());
        }
        else if (menge instanceof MutableSet)
        {
          soDaten.setMengenElemente(menge.getName(), menge.getElements());
          
          AttributeGroup atg = dataModel.getAttributeGroup("atg.dynamischeMenge");
          if (atg != null)
          {  
            Data data = ((ConfigSystemObject)menge).getConfigurationData(atg, lookup);
            if (data != null)
            {
              soDaten.setDynMengenDaten(menge.getName(), data);
            }  
          }  
        }
      }
      
      hoDaten.setSystemObjektDaten(so, soDaten);
    }
    
    _hoDaten.put(ho, hoDaten);
  }
  
  private Map<HierarchieObjekt, HODaten> _hoDaten = new HashMap<HierarchieObjekt, HODaten>();
   
  public IStatus kopiereZwischengespeichertDaten(HierarchieObjekt ho, HierarchieObjekt zwischengespeichertesHO)
  {
    HODaten hoDaten = _hoDaten.get(zwischengespeichertesHO);
    
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten fr Hierarchieobjekt " +
        zwischengespeichertesHO, null);
    
    if (hoDaten != null)
    {
      int index = 0;
      
      for (SystemObject lauf: hoDaten.getSystemObjekte())
      {
        SystemObjektDaten soDaten = hoDaten.getSystemObjektDaten(lauf);
        Assert.isTrue(soDaten != null);
        SystemObject so = ho.getSystemObjekt(index);
        Assert.isTrue(so.getPid().equals(soDaten.getPid()));
        DataModel dataModel = _hm.getHierarchie().getDataModel();
        MultiStatus unterStatus = status;
        
        if (hoDaten.getSystemObjekte().size() > 0)
        {
          unterStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten fr Systemobjekt " +
              soDaten.getPid(), null);
        }
        
        try
        {
          so.setName(soDaten.getName());
          
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Setze Name \"" + soDaten.getName() + "\" fr Systemobjekt "
              + so.getPid()));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Name \"" + soDaten.getName() + "\" fr Systemobjekt "
              + so.getPid() + " konnte nicht gesetzt werden", e));
        }
        
        for (String atgName: soDaten.getAtg())
        {
          AttributeGroup atg = dataModel.getAttributeGroup(atgName);
          if (atg != null)
          {  
            Data data = soDaten.getAtgDaten(atgName);

            try
            {
              so.setConfigurationData(atg, data);
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Kopiere Daten der ATG "
                  + atg.getPid() + " fr Systemobjekt " + so.getPid()));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Daten der ATG "
                  + atg.getPid() + " fr Systemobjekt " + so.getPid() + " konnten nicht kopiert werden", e));
            }
          }
          else
          {
            _debug.warning("Attributgruppe " + atgName + " ist nicht vorhanden");
          }
        }
        
        for (String mengenName: soDaten.getDynMengen())
        {
          ObjectSet menge = ((ConfigurationObject)so).getObjectSet(mengenName);
          if (menge != null)
          {
            AttributeGroup atg = dataModel.getAttributeGroup("atg.dynamischeMenge");
            if (atg != null)
            {  
              Data data = soDaten.getDynMengenDaten(mengenName);
              try
              {
                menge.setConfigurationData(atg, data);
                status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Kopiere Daten der ATG "
                    + atg.getPid() + " fr die Menge "  +  mengenName + " am Systemobjekt " + so.getPid()));
              }
              catch (ConfigurationChangeException e)
              {
                status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Daten der ATG "
                    + atg.getPid() + " fr die Menge "  +  mengenName + " am Systemobjekt " + so.getPid(), e));
              }
            }
            else
            {
              _debug.warning("Attributgruppe atg.dynamischeMenge ist nicht vorhanden");
            }
          }
          else
          {
            status.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "MengeName \"" + mengenName + 
                "\" is am Systemobjekt " + so.getPid() + " nicht vorhanden"));
          }
        }
        
        for (String mengenName: soDaten.getMengen())
        {
          List<SystemObject> elementListe = soDaten.getMengenElemente(mengenName);
          if (elementListe != null && elementListe.size() > 0)
          {
            ObjectSet menge = ((ConfigurationObject)so).getObjectSet(mengenName);
            if (menge != null)
            {
              for (SystemObject element: elementListe)
              {
                try
                {
                  menge.add(element);
                  status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + element.getPid() +
                      " der  Menge "  +  mengenName + " am Systemobjekt " + so.getPid() + " hinzugefgt"));
                }
                catch (ConfigurationChangeException e)
                {
                  status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + element.getPid() +
                      " konnte der  Menge "  +  mengenName + " am Systemobjekt " + so.getPid() + 
                      " nicht hinzugefgt werden ", e));
                }
              }
            }
            else
            {
              status.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "MengeName \"" + mengenName + 
                  "\" is am Systemobjekt " + so.getPid() + " nicht vorhanden"));
            }
          }
        }
        
        index += 1;
        if (unterStatus != status)
        {
          status.add(unterStatus);
        }
      }
    }
    else
    {
      _debug.warning("Fr Hierarchieobjekt " + ho + " wurden keine zwischengespeicherte Daten gefunden");
    }
    
    return status;
  }
  
}