/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

 package de.bsvrz.buv.plugin.tkabasis.zustaende;

 
/**
 * Die Klasse implementiert eine Zustandsmaschine  
 *
 * @version $Revision: 1.3 $ / $Date: 2011/01/17 10:53:46 $ / ($Author: Sans $)
 *
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 *
 */
public class EndlicherAutomat {
     
    /**
     * Die Anzahl der Zustaende in der Zustandsmaschine
     */
    protected int _anzahlZustaende = 0;
       
    /**
     * Der momentane Zustand
     */
    protected Zustand _momentZustand = null;
     
    /**
     * Das Feld der Zustaende
     */
    protected Zustand[] _eaZustaende = null;
    
    /**
     * Konstruktor
     */
    public EndlicherAutomat() {
        this(32);
    }
        
    /**
     * Konstruktor, mit max. Anzahl der Zustaende 
     * 
     * @param maxZustaende
     *          Anzahl der Erlaubten Zustnde
     */
    public EndlicherAutomat(int maxZustaende) {
        _eaZustaende = new Zustand[maxZustaende];
        //numStates = maxStates;
    }
        
    /**
     * Konstruktor, mit initialem Zustand
     * 
     * @param initialZustand
     *          Anfangszustand der Zustandsmaschine
     */
    public EndlicherAutomat(Zustand initialZustand) {
        this();
        addZustand(initialZustand);
        setZustand(initialZustand);
    }
      
    /**
     * Konstruktor, mit max. Anzahl dr Zustnde und initialem Zustand
     * 
     * @param maxZustaende
     *          Anzahl der Erlaubten Zustnde
     * @param initialZustand
     *          Anfangszustand der Zustandsmaschine
     */
    public EndlicherAutomat(int maxZustaende, Zustand initialZustand) {
        this(maxZustaende);
        addZustand(initialZustand);
        setZustand(initialZustand);
    }
    
    
    /**
     * Gibt den momentanen Zustandder Zustandsmaschine zurck
     * @return derzeitiger Zustand
     *        
     */
    public Zustand getMomentZustand() {
        return _momentZustand;
    }
    
    
    /**
     * Fgt einen Zustand hinzu
     * 
     * @param zustand
     *          der Zustand der hinzugefuegt wird
     * @return
     *          Anzahl der derzeitigen Zustaende
     */
    public int addZustand(Zustand zustand) {
      
        if (_anzahlZustaende == _eaZustaende.length)
            throw new IllegalArgumentException("cannot add an additional state.");
        
        _eaZustaende[_anzahlZustaende++] = zustand;
        
        return _anzahlZustaende;
    }
    
    
    /**
     * Setzt den Zustand der Zustandsmaschine
     * 
     * @param zustand
     *          der Zustand
     */
    public void setZustand(Zustand zustand) {
        _momentZustand = zustand;
    }
    
    
    /**
     * Fgt ein Feld von Zustaenden hinzu
     * 
     * @param zustaende
     *          Feld der Zustaende
     * @return
     *          Antahl der Zustaende
     */
    public int addZustaende(Zustand[] zustaende) {
      _anzahlZustaende = zustaende.length;
      _eaZustaende = zustaende;
        
        return _anzahlZustaende;
    }
    
    /**
     * Entfernt alle Zustaende
     */
    public void removeAlleZustaende() {
        // Keep the array allocated, but remove the
        // states.
        for (int i=0; i<_eaZustaende.length; i++)
          _eaZustaende[i] = null;
    }
    
    /**
     * Fgt einen Zustandsbergang zur Zustandsmaschine hinzu
     * 
     * @param start
     *          der Anfangszustand
     * @param uebergang
     *          der Index des bergangs
     * @param end
     *          der Endzustand
     */
    public void addUebergang(Zustand start, int uebergang, Zustand end) {
        start.addUebergang(uebergang, end);
    }

    /**
     * Prft den Zustandsuebergang von einem Zustand zum anderen anhand des 
     * uebergebenen Index. 
     * 
     * @param uebergang 
     *          der Index des Uebergangs
     * @return
     *          Prfung erfolgreich/fehlgeschlagen
     */
    public boolean pruefeUebergang(int uebergang) {
        Zustand neuZustand = _momentZustand.uebergang(uebergang);
        
        if (neuZustand == null)
          return false;
        else
          return true;
    }

    /**
     * Realisert den Zustandsuebergang von einem Zustand zum anderen anhand des 
     * uebergebenen Index. Die Methode liefert false, wenn der 
     * bergang nicht fr den Zustand definiert ist. 
     *
     * @param uebergang 
     *          der Index des Uebergangs
     * @return
     *          bergang erfolgt/nicht erfolgt
     */
    public boolean doUebergang(int uebergang) {
        Zustand neuZustand = _momentZustand.uebergang(uebergang);
        
        if (neuZustand == null)
            return false;
        
        _momentZustand = neuZustand;
        
        return true;
    }
    
    /**
     * Realisert den Zustandsuebergang von einem Zustand zum anderen anhand des 
     * uebergebenen Index. Die Methode erzeugt eine NullPointerException, wenn der 
     * bergang nicht fr den Zustand definiert ist 
     *
     * @param uebergang 
     *          der Index des Uebergangs
     * @return
     *          der neue Zustand der Zustandsmaschine
     */
    public Zustand uebergang(int uebergang) {
        Zustand neuZustand = _momentZustand.uebergang(uebergang);
        
        if (neuZustand == null)
            throw new NullPointerException("transition not defined for this state.");
        
        _momentZustand = neuZustand;
        
        return _momentZustand;
    }
     
    /**
     * Liefert den gewnschten Zustand
     * 
     * @param i 
     *          der Index des Zustands der geholt werden soll
     * @return
     *          Den Zustand des Index
     */
    public Zustand getZustand(int i) {
        return _eaZustaende[i];
    }
    
    public int getAnzahlZustaende()
    {
      return _anzahlZustaende;
    }

    public void setAnzahlZustaende(int anzahlZustaende)
    {
      this._anzahlZustaende = anzahlZustaende;
    }

 
 
    
}
