/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkanba;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.ArrayInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Hierachieobjekt-Typen Netzbeeinflussungsanlagen
 *   
 * Syntax-Aubau:  
 * IHierarchieObjektTyp("Name", new TypInfo[]{<TypInfo>,...})
 * IHierarchieObjektTyp("Name", new TypInfo[]{<TypInfo>,...}, isSystemObjektVorhanden)
 * IHierarchieObjektTyp("Name", uebergeorneterHOT, "typ.oberTyp", "typ.unterTyp", alleTypen)
 *  
 * TypInfo = new TypInfo("typ.pid", new AtgInfo[]{<AtgInfo>,...}, new MengeInfo[]{<MengeInfo>,...})
 * AtgInfo = new AtgInfo("atg.pid", isMandatory, new IAttributInfo[]{<ArrayInfo/AttributInfo>,...})
 * MengeInfo = new MengeInfo("menge.pid", "menge.name", isDynamisch, anwenderAenderbareTypen)
 * ArrayInfo = new ArrayInfo("Name", new IAttributInfo[]{<ArrayInfo/AttributInfo>,...})
 * ArrayInfo = new ArrayInfo("Name", isPflichtfeld, isEindeutig, isAutoGeneriert, isAenderbarNeu, isAenderbarBearb)
 * AttributInfo = new AttributInfo("Name", isPflichtfeld, isEindeutig, isAutoGeneriert, isAenderbarNeu, isAenderbarBearb, isVorbelegt)
 *   
 * @author Krause
 *
 */
public enum HOT_NBA implements IHierarchieObjektTyp
{
  Masche("Masche", new TypInfo[] 
  { 
      new TypInfo("typ.masche",	new AtgInfo[] 
	    {
          new AtgInfo("atg.routen", true, new IAttributInfo[]
          {
              new ArrayInfo("Route", true, false, false, false, false)    
          })
  	  }, 
      null ) // Keine Mengen definiert
  }), 
  ROUTE("Route", new TypInfo[] 
  { 
      new TypInfo("typ.route", new AtgInfo[] 
      {
          new AtgInfo("atg.bestehtAusLinienObjekten", true, new IAttributInfo[]
   	      {
   	        new ArrayInfo("LinienReferenz", true, false, false, false, false)  	   	    	    		
   	      })
   	  }, 
      null ) // Keine Mengen definiert
  }),
  // Die folgenden HOT erben jeweils von ROUTE mit gleichem Typ
  HauptRoute("HauptRoute", ROUTE, "typ.route", "typ.route", false),
  NebenRoute("NebenRoute", ROUTE, "typ.route", "typ.route", false),
  RoutenStueck("RoutenStck", new TypInfo[] 
  { 
      new TypInfo("typ.routenStck", new AtgInfo[] 
      {
          new AtgInfo("atg.bestehtAusLinienObjekten", true, new IAttributInfo[]
          {
              new ArrayInfo("LinienReferenz", true, false, false, false, false)  
          }),
      }, 
      null ) // Keine Mengen definiert
  }),
  ASS("ueresStraenSegment", new TypInfo[] 
  { 
//     new TypInfo("typ.ueresStraenSegment", new AtgInfo[] 
//     {
//         //TODO Workaround um AeueresStraenSegmente anlegen zu knnen,spter wieder raus
//         new AtgInfo("atg.bestehtAusLinienObjekten", false, new IAttributInfo[]
//         {
//             new ArrayInfo("LinienReferenz", true, false, false, true, true)  
//         }),
//     },
      
       new TypInfo("typ.ueresStraenSegment", 
         null,  // Keine Attributgruppen definiert
         null ) // Keine Mengen definiert
  },
  true); 

  
  /**
   * Name des HOT - wird in der Navigationsdarstellung verwendet
   */
  private String _name;
  
  /**
   * Die Systemobjekt-Typen, die von diesem HOT verwendet werden
   */
  private List<String> _typen = new Vector<String>();
  
  /**
   * Die Typinfos zu den Systemobjekttypen
   */
  private Map<String, TypInfo> _typInfos = new HashMap<String, TypInfo>();
  
  /**
   * Flagge die anzeigt ob das SystemObjekt bereits vorhanden ist und nur ausgewhlt werden soll
   */
  private boolean _isSystemObjektVorhanden = false;
    
  /**
   * Erzeugen eines neuen HOT_NBA
   * 
   * @param name Name des HOT
   * @param typInfos Systemobjekt-Typen und zugehrige Informationen, 
   * die von diesem HOT verwendet werden
   */
  private HOT_NBA(String name, TypInfo[] typInfos)
  {
    _name = name;
    if (typInfos != null)
    {  
    	for (TypInfo  info: typInfos)
    	{	
    		_typen.add(info.getTyp());
    		_typInfos.put(info.getTyp(), info);
    	}	
    }  
  }
  
    /**
     * Erzeugen eines neuen HOT_NBA
     * 
     * @param name
     *          Name des HOT
     * @param typInfos
     *          Systemobjekt-Typen und zugehrige Informationen, die von diesem
     *          HOT verwendet werden
     * @param isSystemObjektVorhanden
     *          Flag, ob das SystemObjekt bereits vorhanden ist
     */
    private HOT_NBA( String name, TypInfo[] typInfos, boolean isSystemObjektVorhanden )
    {
      this( name, typInfos );
      _isSystemObjektVorhanden = isSystemObjektVorhanden;
    }
  
  /**
   * Erzeugen eines neuen HOT_NBA auf Basis eines bestehenden HOT_NBA
   * Es werden alle Infos aller Systemobjekttypen des bergeordneten HOT an
   * den neuen HOT bertragen oder nur alle Infos des bergeordneten HOT fr 
   * nur den Systemobjekttyp oberTyp in die neu zu erzeugende Instanz fr den 
   * Sytemobjekttyp unterTyp bernommen.<br>
   * 
   * @param name Name des HOT
   * @param uebergeorneterHOT, der beerbt werden soll
   * @param oberTyp   bergeordneter Typ
   * @param unterTyp  untergeordneter Typ
   * @param alleTypen Flag, um alle Typen oder nur den Typ oberTyp zu bernehmen
   */
  private HOT_NBA(String name, HOT_NBA uebergeorneterHOT, String oberTyp, String unterTyp, boolean alleTypen)
  {
    _name = name;
    
    if (alleTypen)
    { 
      for (String typ: uebergeorneterHOT.getTypen())
      { 
        TypInfo typInfo = uebergeorneterHOT.getTypInfo(typ);
        
        addTypInfos( typInfo, oberTyp, unterTyp );
      }
    }
    else
    {
      TypInfo typInfo = uebergeorneterHOT.getTypInfo(oberTyp);
      
      addTypInfos( typInfo, oberTyp, unterTyp );      
    }
  }

  private void addTypInfos( TypInfo typInfo, String oberTyp, String unterTyp )
  {
   
    List<String> atgs = typInfo.getAtgs();
    AtgInfo atgInfo[] = new AtgInfo[atgs.size()];
    for (int i = 0; i < atgInfo.length; i += 1)
    {
      atgInfo[i] = typInfo.getAtgInfo(atgs.get(i)); 
    }
    
    List<String> mengen = typInfo.getMengen();
    MengeInfo mengeInfo[] = new MengeInfo[mengen.size()];
    for (int i = 0; i < mengeInfo.length; i += 1)
    {
      mengeInfo[i] = typInfo.getMengeInfo(mengen.get(i)); 
    }
    
    String typ = typInfo.getTyp();
    
    if (typ.equals(oberTyp))
    { 
      _typen.add(unterTyp);
      _typInfos.put(unterTyp, new TypInfo(unterTyp, atgInfo, mengeInfo));
    }
    else
    { 
      _typen.add(typ);
      _typInfos.put(typ, new TypInfo(typ, atgInfo, mengeInfo));
    }
  }
  
  /**
   * Text-Reprsentation fr diesen HOT
   * 
   * @return den Namen des HOT - wird in der Navigationsdarstellung verwendet
   */
  @Override
  public String toString()
  {
    return getName();
  }
    
  /**
	 * @see IHierarchieObjektTyp#getName()
	 */
  public String getName()
  {
    return _name;
  }
  
  /**
	 * @see IHierarchieObjektTyp#getSymbolName()
	 */
  public String getSymbolName()
  {
  	return super.toString();
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypen()
	 */
  public List<String> getTypen()
  {
    return Collections.unmodifiableList(_typen); 
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(String)
	 */
  public TypInfo getTypInfo(String typ)
  {
  	return _typInfos.get(typ);
  }

  /**
   * @see IHierarchieObjektTyp#getTypInfo(SystemObjectType)
   */
  public TypInfo getTypInfo(SystemObjectType sot)
  {
    TypInfo result = getTypInfo(sot.getPid());
    
    if (result == null && !sot.isBaseType())
    {
      for (SystemObjectType superTyp: sot.getSuperTypes())
      {
        if (superTyp.isConfigurating())
        {
           result = getTypInfo(superTyp);
           if (result != null)
           {
             break;
           }
        }
      }
    }
    
    return result;
  }
  
  @Override
  public boolean isSystemObjektVorhanden()
  {
    return _isSystemObjektVorhanden;
  }
  
  public boolean isSystemObjektNichtAenderbar()
  {
    return isSystemObjektVorhanden();
  }
  
}	
